
#ifndef MCS_H  /* this keeps this header from getting rolled in more than once */
#define MCS_H

#include <stdio.h>
#include <time.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/un.h>
#include <netinet/in.h> /* for network sockets */
#include <arpa/inet.h>  /* for network sockets */

#define LWA_IP_MSE   "127.0.0.1" /* IP address of MCS Scheduler "ms_exec" process */
#define LWA_PORT_MSE 9734        /* port for MCS Scheduler "ms_exec" process */
#define LWA_PORT_MSE2 9735        /* port for MCS Scheduler "ms_mdre_ip" process */

#define MIB_LABEL_FIELD_LENGTH 32
#define MIB_VAL_FIELD_LENGTH 8192 /* changed from 256 on 100701 for v.0.6.1 */

/* for IP exchanges between MCS/Exec and ms_mdre_ip */
struct LWA_mib_entry {
  char ss[4];                          /* three-character subsystem designator */
  char label[MIB_LABEL_FIELD_LENGTH];  /* MIB label */
  char val[MIB_VAL_FIELD_LENGTH];      /* MIB value, as char. */
  struct timeval last_change;          /* Indicates time this entry was last changed */
                       /* .tv_sec is seconds into current epoch */
                       /* .tv_usec is fractional remainder in microseconds */
  };

/* Using approach of bit-flagging (note power-of-two values) so these can be added: */
#define LWA_MIBERR_OK         0 /* no error to report */
#define LWA_MIBERR_CANTOPEN   1 /* couldn't open MIB dbm */
#define LWA_MIBERR_CANTSTORE  2 /* couldn't store to MIB dbm */
#define LWA_MIBERR_REF_UNK    4 /* REFERENCE was unrecognized, so not sure what this message is */
                                /* in response to.  Other than SUMMARY, MIB may not have been */
                                /* properly updated. */
#define LWA_MIBERR_CANTFETCH  8 /* couldn't fetch from MIB dbm */
#define LWA_MIBERR_SID_UNK   16 /* Subsystem ID (3-char) was unrecognized. Other than SUMMARY, */
                                /* MIB may not have been properly updated. */
#define LWA_MIBERR_SID_CID   32 /* Command is something this subsystem shouldn't have supported */
                                /* e.g., NU# doing something other than PNG, RPT, or SHT */
                                /* e.g., Receiving MCS as a subsystem ID */  
                                /* e.g., Subsystem MIB handler (ms_mcic_XXX.c) didn't recognize command */ 
#define LWA_MIBERR_OTHER     64 /* MIB may be out of sync for other reasons.  For example: */
                                /* - PTQ timeout, so not sure if command was acted up or not */ 

/* Subsystem SUMMARY (MIB 1.1) values */
#define LWA_SIDSUM_NULL    0 
#define LWA_SIDSUM_NORMAL  1
#define LWA_SIDSUM_WARNING 2
#define LWA_SIDSUM_ERROR   3
#define LWA_SIDSUM_BOOTING 4
#define LWA_SIDSUM_SHUTDWN 5

/* === LWA Subsystem IDs === */
#define LWA_MAX_SID 18 /* maximum subsystem code; also maximum number of subsystems */
#define LWA_SID_NU1  1 /* null subsystem #1 (used for testing) */
#define LWA_SID_NU2  2 /* null subsystem #2 (used for testing) */
#define LWA_SID_NU3  3 /* null subsystem #3 (used for testing) */
#define LWA_SID_NU4  4 /* null subsystem #4 (used for testing) */
#define LWA_SID_NU5  5 /* null subsystem #5 (used for testing) */
#define LWA_SID_NU6  6 /* null subsystem #6 (used for testing) */
#define LWA_SID_NU7  7 /* null subsystem #7 (used for testing) */
#define LWA_SID_NU8  8 /* null subsystem #8 (used for testing) */
#define LWA_SID_NU9  9 /* null subsystem #9 (used for testing) */
#define LWA_SID_MCS 10 /* MCS */
#define LWA_SID_SHL 11 /* SHL (shelter) */
#define LWA_SID_ASP 12 /* ASP */
#define LWA_SID_DP_ 13 /* DP */
#define LWA_SID_DR1 14 /* MCS-DR #1 */
#define LWA_SID_DR2 15 /* MCS-DR #2 */
#define LWA_SID_DR3 16 /* MCS-DR #3 */
#define LWA_SID_DR4 17 /* MCS-DR #4 */
#define LWA_SID_DR5 18 /* MCS-DR #5 */
/* When adding subsystems, remember to change LWA_MAX_SID ! */

int LWA_getsid( char *ssc ) {
  /* ssc is the three-character subsystem code */
  /* returns the LWA subsystem ID, or 0 if there is an error */
  int sid = 0;
  if (!strcmp(ssc,"NU1")) sid = LWA_SID_NU1;
  if (!strcmp(ssc,"NU2")) sid = LWA_SID_NU2;
  if (!strcmp(ssc,"NU3")) sid = LWA_SID_NU3;
  if (!strcmp(ssc,"NU4")) sid = LWA_SID_NU4;
  if (!strcmp(ssc,"NU5")) sid = LWA_SID_NU5;
  if (!strcmp(ssc,"NU6")) sid = LWA_SID_NU6;
  if (!strcmp(ssc,"NU7")) sid = LWA_SID_NU7;
  if (!strcmp(ssc,"NU8")) sid = LWA_SID_NU8;
  if (!strcmp(ssc,"NU9")) sid = LWA_SID_NU9;
  if (!strcmp(ssc,"MCS")) sid = LWA_SID_MCS;
  if (!strcmp(ssc,"SHL")) sid = LWA_SID_SHL;
  if (!strcmp(ssc,"ASP")) sid = LWA_SID_ASP;
  if (!strcmp(ssc,"DP_")) sid = LWA_SID_DP_;
  if (!strcmp(ssc,"DR1")) sid = LWA_SID_DR1;
  if (!strcmp(ssc,"DR2")) sid = LWA_SID_DR2;
  if (!strcmp(ssc,"DR3")) sid = LWA_SID_DR3;
  if (!strcmp(ssc,"DR4")) sid = LWA_SID_DR4;
  if (!strcmp(ssc,"DR5")) sid = LWA_SID_DR5;
  return sid;
  } /* LWA_getsid() */
 
/* === LWA Command (TYPE) IDs === */
#define LWA_MAX_CMD  33 /* maximum code; also maximum number of commands */
#define LWA_CMD_MCSSHT 0 /* Not a subsystem command.  Directs ms_mcic to shutdown */
#define LWA_CMD_PNG    1 /* PNG */
#define LWA_CMD_RPT    2 /* RPT */
#define LWA_CMD_SHT    3 /* SHT */
#define LWA_CMD_INI    4 /* INI (SHL,ASP,DP_,DR#) */
#define LWA_CMD_TMP    5 /* TMP (SHL) */
#define LWA_CMD_DIF    6 /* DIF (SHL) */
#define LWA_CMD_PWR    7 /* PWR (SHL) */
#define LWA_CMD_FIL    8 /* FIL (ASP) */
#define LWA_CMD_AT1    9 /* AT1 (ASP) */
#define LWA_CMD_AT2   10 /* AT2 (ASP) */
#define LWA_CMD_ATS   11 /* ATS (ASP) */
#define LWA_CMD_FPW   12 /* FPW (ASP) */
#define LWA_CMD_RXP   13 /* RXP (ASP) */
#define LWA_CMD_FEP   14 /* FEP (ASP) */
#define LWA_CMD_TBW   15 /* TBW (DP_) */
#define LWA_CMD_TBN   16 /* TBN (DP_) */
#define LWA_CMD_DRX   17 /* DRX (DP_) */
#define LWA_CMD_BAM   18 /* BAM (DP_) */
#define LWA_CMD_FST   19 /* FST (DP_) */
#define LWA_CMD_CLK   20 /* CLK (DP_) */
#define LWA_CMD_REC   21 /* REC (DR#) */
#define LWA_CMD_DEL   22 /* DEL (DR#) */
#define LWA_CMD_STP   23 /* STP (DR#) */
#define LWA_CMD_GET   24 /* GET (DR#) */
#define LWA_CMD_CPY   25 /* CPY (DR#) */
#define LWA_CMD_DMP   26 /* DMP (DR#) */
#define LWA_CMD_FMT   27 /* FMT (DR#) */
#define LWA_CMD_DWN   28 /* DWN (DR#) */
#define LWA_CMD_UP_   29 /* UP_ (DR#).  Note ICD calls this "UP" (no underscore) */
#define LWA_CMD_SEL   30 /* SEL (DR#) */
#define LWA_CMD_SYN   31 /* SYN (DR#) */
#define LWA_CMD_TST   32 /* TST (DR#) */
#define LWA_CMD_BUF   33 /* BUF (DR#) */


/* When adding commands, remember to change LWA_MAX_CMD ! */

int LWA_getcmd( char *ssc ) {
  /* ssc is the three-character command (TYP) */
  /* returns the LWA command ("TYPE"), or 0 if there is an error */
  int cmd = 0;
  if (!strcmp(ssc,"PNG")) cmd = LWA_CMD_PNG;
  if (!strcmp(ssc,"RPT")) cmd = LWA_CMD_RPT;
  if (!strcmp(ssc,"SHT")) cmd = LWA_CMD_SHT;
  if (!strcmp(ssc,"INI")) cmd = LWA_CMD_INI;
  if (!strcmp(ssc,"TMP")) cmd = LWA_CMD_TMP;
  if (!strcmp(ssc,"DIF")) cmd = LWA_CMD_DIF;
  if (!strcmp(ssc,"PWR")) cmd = LWA_CMD_PWR;
  if (!strcmp(ssc,"FIL")) cmd = LWA_CMD_FIL;
  if (!strcmp(ssc,"AT1")) cmd = LWA_CMD_AT1;
  if (!strcmp(ssc,"AT2")) cmd = LWA_CMD_AT2;
  if (!strcmp(ssc,"ATS")) cmd = LWA_CMD_ATS;
  if (!strcmp(ssc,"FPW")) cmd = LWA_CMD_FPW;
  if (!strcmp(ssc,"RXP")) cmd = LWA_CMD_RXP;
  if (!strcmp(ssc,"FEP")) cmd = LWA_CMD_FEP;
  if (!strcmp(ssc,"TBW")) cmd = LWA_CMD_TBW;
  if (!strcmp(ssc,"TBN")) cmd = LWA_CMD_TBN;
  if (!strcmp(ssc,"DRX")) cmd = LWA_CMD_DRX;
  if (!strcmp(ssc,"BAM")) cmd = LWA_CMD_BAM;
  if (!strcmp(ssc,"FST")) cmd = LWA_CMD_FST;
  if (!strcmp(ssc,"CLK")) cmd = LWA_CMD_CLK;
  if (!strcmp(ssc,"REC")) cmd = LWA_CMD_REC;
  if (!strcmp(ssc,"DEL")) cmd = LWA_CMD_DEL;
  if (!strcmp(ssc,"STP")) cmd = LWA_CMD_STP;
  if (!strcmp(ssc,"GET")) cmd = LWA_CMD_GET;
  if (!strcmp(ssc,"CPY")) cmd = LWA_CMD_CPY;
  if (!strcmp(ssc,"DMP")) cmd = LWA_CMD_DMP;
  if (!strcmp(ssc,"FMT")) cmd = LWA_CMD_FMT;
  if (!strcmp(ssc,"DWN")) cmd = LWA_CMD_DWN;
  if (!strcmp(ssc,"UP_")) cmd = LWA_CMD_UP_;
  if (!strcmp(ssc,"SEL")) cmd = LWA_CMD_SEL;
  if (!strcmp(ssc,"SYN")) cmd = LWA_CMD_SYN;
  if (!strcmp(ssc,"TST")) cmd = LWA_CMD_TST;
  if (!strcmp(ssc,"BUF")) cmd = LWA_CMD_BUF;
  return cmd;
  } /* LWA_getcmd() */

/* This has to match up with the definition for MCS/Scheduler */
#define LWA_CMD_STRUCT_DATA_FIELD_LENGTH 256
struct LWA_cmd_struct {
  long int sid;      /* subsystem ID.  Must be "long int" to accomodate message queue use */
  long int ref;      /* REFERENCE number */
  int  cid;          /* command ID */
  //int  subslot;      /* subslot for  */
  int  bScheduled;   /* = 0 means "not scheduled (do as time permits)"; */
                     /* = 1 means "do as close as posible to time indicated by tv field" */  
  struct timeval tv; /* Indicates time this command is to take effect */
                     /* .tv_sec is seconds into current epoch */
                     /* .tv_usec is fractional remainder in microseconds */
  int  bAccept;      /* response: see LWA_MSELOG_TP_* */
  int  eSummary;     /* summary; see LWA_SIDSUM_* macrodefines */
  int  eMIBerror;    /* >0 if ms_mcic had a problem with the MIB.  see LWA_MIBERR_* */
  char data[LWA_CMD_STRUCT_DATA_FIELD_LENGTH]; /* DATA on way out, R-COMMENT on way back */   
  int  datalen;      /* -1 for (printable) string; 0 for zero-length; otherwise number of significant bytes */
  };


void LWA_raw2hex( char *raw, char *hex, int n ) {
  /* creates a string hex[ 0 .. 2*n ] (including \0) which is the printable */
  /* hex represenation of the bytes raw[ 0 .. n-1 ] */

  int i;
  for (i=0; i<n; i++) {
    sprintf( &(hex[2*i]), "%02x", (unsigned char) raw[i] );  
    }
  hex[2*n] = '\0';   

  return;
  } /* LWA_raw2hex() */

void LWA_timeval( 
                 struct timeval *tv, /* time as a timeval struct */
                 long int *mjd,      /* MJD */
                 long int *mpm       /* MPM */  
                 ) {
  /* converts a timeval to MJD and MPM */
  struct tm *tm;      /* from sys/time.h */
  long int a,y,m,p,q;
  
  tm = gmtime(&(tv->tv_sec));
  //printf("LWA_timeval: %02d:%02d:%02d %ld\n", tm->tm_hour, tm->tm_min, tm->tm_sec,           
  //                                            tv->tv_usec);

  /* construct the MJD field */
  /* adapted from http://paste.lisp.org/display/73536 */
  /* can check result using http://www.csgnetwork.com/julianmodifdateconv.html */
  a = (14 - tm->tm_mon) / 12; 
  y = ( tm->tm_year + 1900) + 4800 - a; // tm->tm_year is the number of years since 1900
  m = ( tm->tm_mon + 1) + (12 * a) - 3; // tm->tm_mon is number of months since Jan (0..11)
  p = tm->tm_mday + (((153 * m) + 2) / 5) + (365 * y); 
  q = (y/4) - (y/100) + (y/400) - 32045; 
  *mjd = (p+q) - 2400000.5;

  *mpm = ( (tm->tm_hour)*3600 + (tm->tm_min)*60 + (tm->tm_sec) ) * 1000
                    + (tv->tv_usec)/1000 ; 

  return;
  } /* LWA_timeval() */

void LWA_time( 
               long int *mjd,      /* MJD */
               long int *mpm       /* MPM */  
               ) {
  /* gets current time; returns MJD and MPM */

  struct timeval tv;  /* from sys/time.h */
  struct timezone tz; /* set but useless; see notes below */

  gettimeofday( &tv, &tz );     /* tz is set, but useless; see notes below */
  LWA_timeval( &tv, mjd, mpm ); /* determine MPM, MJD */
  //printf("%ld %ld\n",mjd,mpm);
  
  return;
  } /* LWA_time() */





/*============================================*/

#endif // #ifndef MCS_H 



